/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2013 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */

@import '~patternfly/dist/css/patternfly-additions.min.css';

.installed-products .list-view-pf-left .fa.list-view-pf-icon-sm {
    border: none;
}

.subscription-status-ct label {
    font-size: 13px;
}

.subscription-status-ct button {
    margin-left: 12px;
}

.subscription-status-ct .value {
    font-weight: 400;
    margin-left: 4px;
}

#register_dialog .modal-footer .alert{
    text-align: left;
    padding-left: 20px;
}

#register_dialog .modal-footer .alert .fa-exclamation-triangle {
    margin-right: 4px;
}

.container-fluid.alert {
    margin: 20px;
}

#subscription-register-url {
    margin-bottom: 3px;
}

#subscriptions-registering > span {
    margin-left: 7px;
}

.form-tr-ct-title  {
    text-align: right;
    white-space: nowrap;
    color: #888888;
    width: 5px; /* will be expanded by nowrap */
    padding-right: 7px;
}

.form-td-ct-subscriptions {
    width: 80px;
}

#subscription-proxy-use {
    vertical-align: sub;
}

#subscription-register-proxy-form {
    margin-bottom: 4px;
}

.ct-form-layout .ct-form-layout {
    background: #f4f4f4;
    border-width: 1px;
    border-style: solid;
    border-color: #bababa;
    padding: 4px;
    padding-top: 10px;
    width: 100%;
}

/* Make a dialog visible */
.dialog-ct-visible {
    display: block;
}

